//  SPDX-FileCopyrightText: 2023 Yury Gubich <blue@macaw.me>
//  SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <map>
#include <functional>
#include <string>
#include <memory>

#include "request/request.h"
#include "response/response.h"
#include "handler/handler.h"

class Server;

class Router {
    using Handler = std::unique_ptr<Handler::Handler>;
public:
    Router();

    void addRoute(Handler handler);
    void route(const std::string& path, std::unique_ptr<Request> request);

private:
    void handleNotFound(const std::string& path, std::unique_ptr<Request> request);
    void handleInternalError(const std::string& path, const std::exception& exception, std::unique_ptr<Request> request);
    void handleMethodNotAllowed(const std::string& path, std::unique_ptr<Request> request);

private:
    std::map<std::string, Handler> get;
    std::map<std::string, Handler> post;

};
